/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_PPC_ENUMS_H
#define LLVM_IR_INTRINSIC_PPC_ENUMS_H
namespace llvm::Intrinsic {
enum PPCIntrinsics : unsigned {
// Enum values for intrinsics.
    ppc_addex = 11163,                                 // llvm.ppc.addex (IntrinsicsPowerPC.td:2069)
    ppc_addf128_round_to_odd,                  // llvm.ppc.addf128.round.to.odd (IntrinsicsPowerPC.td:119)
    ppc_addg6s,                                // llvm.ppc.addg6s (IntrinsicsPowerPC.td:682)
    ppc_addg6sd,                               // llvm.ppc.addg6sd (IntrinsicsPowerPC.td:676)
    ppc_altivec_crypto_vcipher,                // llvm.ppc.altivec.crypto.vcipher (IntrinsicsPowerPC.td:1280)
    ppc_altivec_crypto_vcipherlast,            // llvm.ppc.altivec.crypto.vcipherlast (IntrinsicsPowerPC.td:1282)
    ppc_altivec_crypto_vncipher,               // llvm.ppc.altivec.crypto.vncipher (IntrinsicsPowerPC.td:1284)
    ppc_altivec_crypto_vncipherlast,           // llvm.ppc.altivec.crypto.vncipherlast (IntrinsicsPowerPC.td:1286)
    ppc_altivec_crypto_vpermxor,               // llvm.ppc.altivec.crypto.vpermxor (IntrinsicsPowerPC.td:1260)
    ppc_altivec_crypto_vpermxor_be,            // llvm.ppc.altivec.crypto.vpermxor.be (IntrinsicsPowerPC.td:1264)
    ppc_altivec_crypto_vpmsumb,                // llvm.ppc.altivec.crypto.vpmsumb (IntrinsicsPowerPC.td:1288)
    ppc_altivec_crypto_vpmsumd,                // llvm.ppc.altivec.crypto.vpmsumd (IntrinsicsPowerPC.td:1294)
    ppc_altivec_crypto_vpmsumh,                // llvm.ppc.altivec.crypto.vpmsumh (IntrinsicsPowerPC.td:1290)
    ppc_altivec_crypto_vpmsumw,                // llvm.ppc.altivec.crypto.vpmsumw (IntrinsicsPowerPC.td:1292)
    ppc_altivec_crypto_vsbox,                  // llvm.ppc.altivec.crypto.vsbox (IntrinsicsPowerPC.td:1257)
    ppc_altivec_crypto_vshasigmad,             // llvm.ppc.altivec.crypto.vshasigmad (IntrinsicsPowerPC.td:1269)
    ppc_altivec_crypto_vshasigmaw,             // llvm.ppc.altivec.crypto.vshasigmaw (IntrinsicsPowerPC.td:1274)
    ppc_altivec_dss,                           // llvm.ppc.altivec.dss (IntrinsicsPowerPC.td:401)
    ppc_altivec_dssall,                        // llvm.ppc.altivec.dssall (IntrinsicsPowerPC.td:403)
    ppc_altivec_dst,                           // llvm.ppc.altivec.dst (IntrinsicsPowerPC.td:405)
    ppc_altivec_dstst,                         // llvm.ppc.altivec.dstst (IntrinsicsPowerPC.td:413)
    ppc_altivec_dststt,                        // llvm.ppc.altivec.dststt (IntrinsicsPowerPC.td:417)
    ppc_altivec_dstt,                          // llvm.ppc.altivec.dstt (IntrinsicsPowerPC.td:409)
    ppc_altivec_lvebx,                         // llvm.ppc.altivec.lvebx (IntrinsicsPowerPC.td:437)
    ppc_altivec_lvehx,                         // llvm.ppc.altivec.lvehx (IntrinsicsPowerPC.td:440)
    ppc_altivec_lvewx,                         // llvm.ppc.altivec.lvewx (IntrinsicsPowerPC.td:443)
    ppc_altivec_lvsl,                          // llvm.ppc.altivec.lvsl (IntrinsicsPowerPC.td:1228)
    ppc_altivec_lvsr,                          // llvm.ppc.altivec.lvsr (IntrinsicsPowerPC.td:1230)
    ppc_altivec_lvx,                           // llvm.ppc.altivec.lvx (IntrinsicsPowerPC.td:431)
    ppc_altivec_lvxl,                          // llvm.ppc.altivec.lvxl (IntrinsicsPowerPC.td:434)
    ppc_altivec_mfvscr,                        // llvm.ppc.altivec.mfvscr (IntrinsicsPowerPC.td:423)
    ppc_altivec_mtvscr,                        // llvm.ppc.altivec.mtvscr (IntrinsicsPowerPC.td:425)
    ppc_altivec_mtvsrbm,                       // llvm.ppc.altivec.mtvsrbm (IntrinsicsPowerPC.td:748)
    ppc_altivec_mtvsrdm,                       // llvm.ppc.altivec.mtvsrdm (IntrinsicsPowerPC.td:754)
    ppc_altivec_mtvsrhm,                       // llvm.ppc.altivec.mtvsrhm (IntrinsicsPowerPC.td:750)
    ppc_altivec_mtvsrqm,                       // llvm.ppc.altivec.mtvsrqm (IntrinsicsPowerPC.td:756)
    ppc_altivec_mtvsrwm,                       // llvm.ppc.altivec.mtvsrwm (IntrinsicsPowerPC.td:752)
    ppc_altivec_stvebx,                        // llvm.ppc.altivec.stvebx (IntrinsicsPowerPC.td:455)
    ppc_altivec_stvehx,                        // llvm.ppc.altivec.stvehx (IntrinsicsPowerPC.td:458)
    ppc_altivec_stvewx,                        // llvm.ppc.altivec.stvewx (IntrinsicsPowerPC.td:461)
    ppc_altivec_stvx,                          // llvm.ppc.altivec.stvx (IntrinsicsPowerPC.td:449)
    ppc_altivec_stvxl,                         // llvm.ppc.altivec.stvxl (IntrinsicsPowerPC.td:452)
    ppc_altivec_vabsdub,                       // llvm.ppc.altivec.vabsdub (IntrinsicsPowerPC.td:1298)
    ppc_altivec_vabsduh,                       // llvm.ppc.altivec.vabsduh (IntrinsicsPowerPC.td:1299)
    ppc_altivec_vabsduw,                       // llvm.ppc.altivec.vabsduw (IntrinsicsPowerPC.td:1300)
    ppc_altivec_vaddcuq,                       // llvm.ppc.altivec.vaddcuq (IntrinsicsPowerPC.td:957)
    ppc_altivec_vaddcuw,                       // llvm.ppc.altivec.vaddcuw (IntrinsicsPowerPC.td:956)
    ppc_altivec_vaddecuq,                      // llvm.ppc.altivec.vaddecuq (IntrinsicsPowerPC.td:1175)
    ppc_altivec_vaddeuqm,                      // llvm.ppc.altivec.vaddeuqm (IntrinsicsPowerPC.td:1171)
    ppc_altivec_vaddsbs,                       // llvm.ppc.altivec.vaddsbs (IntrinsicsPowerPC.td:951)
    ppc_altivec_vaddshs,                       // llvm.ppc.altivec.vaddshs (IntrinsicsPowerPC.td:953)
    ppc_altivec_vaddsws,                       // llvm.ppc.altivec.vaddsws (IntrinsicsPowerPC.td:955)
    ppc_altivec_vaddubs,                       // llvm.ppc.altivec.vaddubs (IntrinsicsPowerPC.td:950)
    ppc_altivec_vadduhs,                       // llvm.ppc.altivec.vadduhs (IntrinsicsPowerPC.td:952)
    ppc_altivec_vadduws,                       // llvm.ppc.altivec.vadduws (IntrinsicsPowerPC.td:954)
    ppc_altivec_vavgsb,                        // llvm.ppc.altivec.vavgsb (IntrinsicsPowerPC.td:920)
    ppc_altivec_vavgsh,                        // llvm.ppc.altivec.vavgsh (IntrinsicsPowerPC.td:921)
    ppc_altivec_vavgsw,                        // llvm.ppc.altivec.vavgsw (IntrinsicsPowerPC.td:922)
    ppc_altivec_vavgub,                        // llvm.ppc.altivec.vavgub (IntrinsicsPowerPC.td:923)
    ppc_altivec_vavguh,                        // llvm.ppc.altivec.vavguh (IntrinsicsPowerPC.td:924)
    ppc_altivec_vavguw,                        // llvm.ppc.altivec.vavguw (IntrinsicsPowerPC.td:925)
    ppc_altivec_vbpermd,                       // llvm.ppc.altivec.vbpermd (IntrinsicsPowerPC.td:1244)
    ppc_altivec_vbpermq,                       // llvm.ppc.altivec.vbpermq (IntrinsicsPowerPC.td:1241)
    ppc_altivec_vcfsx,                         // llvm.ppc.altivec.vcfsx (IntrinsicsPowerPC.td:1148)
    ppc_altivec_vcfuged,                       // llvm.ppc.altivec.vcfuged (IntrinsicsPowerPC.td:791)
    ppc_altivec_vcfux,                         // llvm.ppc.altivec.vcfux (IntrinsicsPowerPC.td:1151)
    ppc_altivec_vclrlb,                        // llvm.ppc.altivec.vclrlb (IntrinsicsPowerPC.td:801)
    ppc_altivec_vclrrb,                        // llvm.ppc.altivec.vclrrb (IntrinsicsPowerPC.td:804)
    ppc_altivec_vclzdm,                        // llvm.ppc.altivec.vclzdm (IntrinsicsPowerPC.td:1191)
    ppc_altivec_vclzlsbb,                      // llvm.ppc.altivec.vclzlsbb (IntrinsicsPowerPC.td:652)
    ppc_altivec_vcmpbfp,                       // llvm.ppc.altivec.vcmpbfp (IntrinsicsPowerPC.td:466)
    ppc_altivec_vcmpbfp_p,                     // llvm.ppc.altivec.vcmpbfp.p (IntrinsicsPowerPC.td:560)
    ppc_altivec_vcmpeqfp,                      // llvm.ppc.altivec.vcmpeqfp (IntrinsicsPowerPC.td:469)
    ppc_altivec_vcmpeqfp_p,                    // llvm.ppc.altivec.vcmpeqfp.p (IntrinsicsPowerPC.td:564)
    ppc_altivec_vcmpequb,                      // llvm.ppc.altivec.vcmpequb (IntrinsicsPowerPC.td:521)
    ppc_altivec_vcmpequb_p,                    // llvm.ppc.altivec.vcmpequb.p (IntrinsicsPowerPC.td:632)
    ppc_altivec_vcmpequd,                      // llvm.ppc.altivec.vcmpequd (IntrinsicsPowerPC.td:479)
    ppc_altivec_vcmpequd_p,                    // llvm.ppc.altivec.vcmpequd.p (IntrinsicsPowerPC.td:577)
    ppc_altivec_vcmpequh,                      // llvm.ppc.altivec.vcmpequh (IntrinsicsPowerPC.td:505)
    ppc_altivec_vcmpequh_p,                    // llvm.ppc.altivec.vcmpequh.p (IntrinsicsPowerPC.td:611)
    ppc_altivec_vcmpequq,                      // llvm.ppc.altivec.vcmpequq (IntrinsicsPowerPC.td:537)
    ppc_altivec_vcmpequq_p,                    // llvm.ppc.altivec.vcmpequq.p (IntrinsicsPowerPC.td:546)
    ppc_altivec_vcmpequw,                      // llvm.ppc.altivec.vcmpequw (IntrinsicsPowerPC.td:489)
    ppc_altivec_vcmpequw_p,                    // llvm.ppc.altivec.vcmpequw.p (IntrinsicsPowerPC.td:590)
    ppc_altivec_vcmpgefp,                      // llvm.ppc.altivec.vcmpgefp (IntrinsicsPowerPC.td:472)
    ppc_altivec_vcmpgefp_p,                    // llvm.ppc.altivec.vcmpgefp.p (IntrinsicsPowerPC.td:568)
    ppc_altivec_vcmpgtfp,                      // llvm.ppc.altivec.vcmpgtfp (IntrinsicsPowerPC.td:475)
    ppc_altivec_vcmpgtfp_p,                    // llvm.ppc.altivec.vcmpgtfp.p (IntrinsicsPowerPC.td:572)
    ppc_altivec_vcmpgtsb,                      // llvm.ppc.altivec.vcmpgtsb (IntrinsicsPowerPC.td:524)
    ppc_altivec_vcmpgtsb_p,                    // llvm.ppc.altivec.vcmpgtsb.p (IntrinsicsPowerPC.td:636)
    ppc_altivec_vcmpgtsd,                      // llvm.ppc.altivec.vcmpgtsd (IntrinsicsPowerPC.td:482)
    ppc_altivec_vcmpgtsd_p,                    // llvm.ppc.altivec.vcmpgtsd.p (IntrinsicsPowerPC.td:581)
    ppc_altivec_vcmpgtsh,                      // llvm.ppc.altivec.vcmpgtsh (IntrinsicsPowerPC.td:508)
    ppc_altivec_vcmpgtsh_p,                    // llvm.ppc.altivec.vcmpgtsh.p (IntrinsicsPowerPC.td:615)
    ppc_altivec_vcmpgtsq,                      // llvm.ppc.altivec.vcmpgtsq (IntrinsicsPowerPC.td:540)
    ppc_altivec_vcmpgtsq_p,                    // llvm.ppc.altivec.vcmpgtsq.p (IntrinsicsPowerPC.td:550)
    ppc_altivec_vcmpgtsw,                      // llvm.ppc.altivec.vcmpgtsw (IntrinsicsPowerPC.td:492)
    ppc_altivec_vcmpgtsw_p,                    // llvm.ppc.altivec.vcmpgtsw.p (IntrinsicsPowerPC.td:594)
    ppc_altivec_vcmpgtub,                      // llvm.ppc.altivec.vcmpgtub (IntrinsicsPowerPC.td:527)
    ppc_altivec_vcmpgtub_p,                    // llvm.ppc.altivec.vcmpgtub.p (IntrinsicsPowerPC.td:640)
    ppc_altivec_vcmpgtud,                      // llvm.ppc.altivec.vcmpgtud (IntrinsicsPowerPC.td:485)
    ppc_altivec_vcmpgtud_p,                    // llvm.ppc.altivec.vcmpgtud.p (IntrinsicsPowerPC.td:585)
    ppc_altivec_vcmpgtuh,                      // llvm.ppc.altivec.vcmpgtuh (IntrinsicsPowerPC.td:511)
    ppc_altivec_vcmpgtuh_p,                    // llvm.ppc.altivec.vcmpgtuh.p (IntrinsicsPowerPC.td:619)
    ppc_altivec_vcmpgtuq,                      // llvm.ppc.altivec.vcmpgtuq (IntrinsicsPowerPC.td:543)
    ppc_altivec_vcmpgtuq_p,                    // llvm.ppc.altivec.vcmpgtuq.p (IntrinsicsPowerPC.td:554)
    ppc_altivec_vcmpgtuw,                      // llvm.ppc.altivec.vcmpgtuw (IntrinsicsPowerPC.td:495)
    ppc_altivec_vcmpgtuw_p,                    // llvm.ppc.altivec.vcmpgtuw.p (IntrinsicsPowerPC.td:598)
    ppc_altivec_vcmpneb,                       // llvm.ppc.altivec.vcmpneb (IntrinsicsPowerPC.td:530)
    ppc_altivec_vcmpneb_p,                     // llvm.ppc.altivec.vcmpneb.p (IntrinsicsPowerPC.td:644)
    ppc_altivec_vcmpneh,                       // llvm.ppc.altivec.vcmpneh (IntrinsicsPowerPC.td:514)
    ppc_altivec_vcmpneh_p,                     // llvm.ppc.altivec.vcmpneh.p (IntrinsicsPowerPC.td:623)
    ppc_altivec_vcmpnew,                       // llvm.ppc.altivec.vcmpnew (IntrinsicsPowerPC.td:498)
    ppc_altivec_vcmpnew_p,                     // llvm.ppc.altivec.vcmpnew.p (IntrinsicsPowerPC.td:602)
    ppc_altivec_vcmpnezb,                      // llvm.ppc.altivec.vcmpnezb (IntrinsicsPowerPC.td:533)
    ppc_altivec_vcmpnezb_p,                    // llvm.ppc.altivec.vcmpnezb.p (IntrinsicsPowerPC.td:648)
    ppc_altivec_vcmpnezh,                      // llvm.ppc.altivec.vcmpnezh (IntrinsicsPowerPC.td:517)
    ppc_altivec_vcmpnezh_p,                    // llvm.ppc.altivec.vcmpnezh.p (IntrinsicsPowerPC.td:627)
    ppc_altivec_vcmpnezw,                      // llvm.ppc.altivec.vcmpnezw (IntrinsicsPowerPC.td:501)
    ppc_altivec_vcmpnezw_p,                    // llvm.ppc.altivec.vcmpnezw.p (IntrinsicsPowerPC.td:606)
    ppc_altivec_vcntmbb,                       // llvm.ppc.altivec.vcntmbb (IntrinsicsPowerPC.td:734)
    ppc_altivec_vcntmbd,                       // llvm.ppc.altivec.vcntmbd (IntrinsicsPowerPC.td:743)
    ppc_altivec_vcntmbh,                       // llvm.ppc.altivec.vcntmbh (IntrinsicsPowerPC.td:737)
    ppc_altivec_vcntmbw,                       // llvm.ppc.altivec.vcntmbw (IntrinsicsPowerPC.td:740)
    ppc_altivec_vctsxs,                        // llvm.ppc.altivec.vctsxs (IntrinsicsPowerPC.td:1154)
    ppc_altivec_vctuxs,                        // llvm.ppc.altivec.vctuxs (IntrinsicsPowerPC.td:1157)
    ppc_altivec_vctzdm,                        // llvm.ppc.altivec.vctzdm (IntrinsicsPowerPC.td:1194)
    ppc_altivec_vctzlsbb,                      // llvm.ppc.altivec.vctzlsbb (IntrinsicsPowerPC.td:654)
    ppc_altivec_vdivesd,                       // llvm.ppc.altivec.vdivesd (IntrinsicsPowerPC.td:1330)
    ppc_altivec_vdivesq,                       // llvm.ppc.altivec.vdivesq (IntrinsicsPowerPC.td:1332)
    ppc_altivec_vdivesw,                       // llvm.ppc.altivec.vdivesw (IntrinsicsPowerPC.td:1328)
    ppc_altivec_vdiveud,                       // llvm.ppc.altivec.vdiveud (IntrinsicsPowerPC.td:1331)
    ppc_altivec_vdiveuq,                       // llvm.ppc.altivec.vdiveuq (IntrinsicsPowerPC.td:1333)
    ppc_altivec_vdiveuw,                       // llvm.ppc.altivec.vdiveuw (IntrinsicsPowerPC.td:1329)
    ppc_altivec_vexpandbm,                     // llvm.ppc.altivec.vexpandbm (IntrinsicsPowerPC.td:722)
    ppc_altivec_vexpanddm,                     // llvm.ppc.altivec.vexpanddm (IntrinsicsPowerPC.td:728)
    ppc_altivec_vexpandhm,                     // llvm.ppc.altivec.vexpandhm (IntrinsicsPowerPC.td:724)
    ppc_altivec_vexpandqm,                     // llvm.ppc.altivec.vexpandqm (IntrinsicsPowerPC.td:730)
    ppc_altivec_vexpandwm,                     // llvm.ppc.altivec.vexpandwm (IntrinsicsPowerPC.td:726)
    ppc_altivec_vexptefp,                      // llvm.ppc.altivec.vexptefp (IntrinsicsPowerPC.td:1249)
    ppc_altivec_vextddvlx,                     // llvm.ppc.altivec.vextddvlx (IntrinsicsPowerPC.td:909)
    ppc_altivec_vextddvrx,                     // llvm.ppc.altivec.vextddvrx (IntrinsicsPowerPC.td:913)
    ppc_altivec_vextdubvlx,                    // llvm.ppc.altivec.vextdubvlx (IntrinsicsPowerPC.td:885)
    ppc_altivec_vextdubvrx,                    // llvm.ppc.altivec.vextdubvrx (IntrinsicsPowerPC.td:889)
    ppc_altivec_vextduhvlx,                    // llvm.ppc.altivec.vextduhvlx (IntrinsicsPowerPC.td:893)
    ppc_altivec_vextduhvrx,                    // llvm.ppc.altivec.vextduhvrx (IntrinsicsPowerPC.td:897)
    ppc_altivec_vextduwvlx,                    // llvm.ppc.altivec.vextduwvlx (IntrinsicsPowerPC.td:901)
    ppc_altivec_vextduwvrx,                    // llvm.ppc.altivec.vextduwvrx (IntrinsicsPowerPC.td:905)
    ppc_altivec_vextractbm,                    // llvm.ppc.altivec.vextractbm (IntrinsicsPowerPC.td:710)
    ppc_altivec_vextractdm,                    // llvm.ppc.altivec.vextractdm (IntrinsicsPowerPC.td:716)
    ppc_altivec_vextracthm,                    // llvm.ppc.altivec.vextracthm (IntrinsicsPowerPC.td:712)
    ppc_altivec_vextractqm,                    // llvm.ppc.altivec.vextractqm (IntrinsicsPowerPC.td:718)
    ppc_altivec_vextractwm,                    // llvm.ppc.altivec.vextractwm (IntrinsicsPowerPC.td:714)
    ppc_altivec_vextsb2d,                      // llvm.ppc.altivec.vextsb2d (IntrinsicsPowerPC.td:1077)
    ppc_altivec_vextsb2w,                      // llvm.ppc.altivec.vextsb2w (IntrinsicsPowerPC.td:1075)
    ppc_altivec_vextsd2q,                      // llvm.ppc.altivec.vextsd2q (IntrinsicsPowerPC.td:1085)
    ppc_altivec_vextsh2d,                      // llvm.ppc.altivec.vextsh2d (IntrinsicsPowerPC.td:1081)
    ppc_altivec_vextsh2w,                      // llvm.ppc.altivec.vextsh2w (IntrinsicsPowerPC.td:1079)
    ppc_altivec_vextsw2d,                      // llvm.ppc.altivec.vextsw2d (IntrinsicsPowerPC.td:1083)
    ppc_altivec_vgbbd,                         // llvm.ppc.altivec.vgbbd (IntrinsicsPowerPC.td:1239)
    ppc_altivec_vgnb,                          // llvm.ppc.altivec.vgnb (IntrinsicsPowerPC.td:796)
    ppc_altivec_vinsblx,                       // llvm.ppc.altivec.vinsblx (IntrinsicsPowerPC.td:819)
    ppc_altivec_vinsbrx,                       // llvm.ppc.altivec.vinsbrx (IntrinsicsPowerPC.td:823)
    ppc_altivec_vinsbvlx,                      // llvm.ppc.altivec.vinsbvlx (IntrinsicsPowerPC.td:851)
    ppc_altivec_vinsbvrx,                      // llvm.ppc.altivec.vinsbvrx (IntrinsicsPowerPC.td:855)
    ppc_altivec_vinsd,                         // llvm.ppc.altivec.vinsd (IntrinsicsPowerPC.td:880)
    ppc_altivec_vinsdlx,                       // llvm.ppc.altivec.vinsdlx (IntrinsicsPowerPC.td:843)
    ppc_altivec_vinsdrx,                       // llvm.ppc.altivec.vinsdrx (IntrinsicsPowerPC.td:847)
    ppc_altivec_vinshlx,                       // llvm.ppc.altivec.vinshlx (IntrinsicsPowerPC.td:827)
    ppc_altivec_vinshrx,                       // llvm.ppc.altivec.vinshrx (IntrinsicsPowerPC.td:831)
    ppc_altivec_vinshvlx,                      // llvm.ppc.altivec.vinshvlx (IntrinsicsPowerPC.td:859)
    ppc_altivec_vinshvrx,                      // llvm.ppc.altivec.vinshvrx (IntrinsicsPowerPC.td:863)
    ppc_altivec_vinsw,                         // llvm.ppc.altivec.vinsw (IntrinsicsPowerPC.td:876)
    ppc_altivec_vinswlx,                       // llvm.ppc.altivec.vinswlx (IntrinsicsPowerPC.td:835)
    ppc_altivec_vinswrx,                       // llvm.ppc.altivec.vinswrx (IntrinsicsPowerPC.td:839)
    ppc_altivec_vinswvlx,                      // llvm.ppc.altivec.vinswvlx (IntrinsicsPowerPC.td:867)
    ppc_altivec_vinswvrx,                      // llvm.ppc.altivec.vinswvrx (IntrinsicsPowerPC.td:871)
    ppc_altivec_vlogefp,                       // llvm.ppc.altivec.vlogefp (IntrinsicsPowerPC.td:1250)
    ppc_altivec_vmaddfp,                       // llvm.ppc.altivec.vmaddfp (IntrinsicsPowerPC.td:980)
    ppc_altivec_vmaxfp,                        // llvm.ppc.altivec.vmaxfp (IntrinsicsPowerPC.td:928)
    ppc_altivec_vmaxsb,                        // llvm.ppc.altivec.vmaxsb (IntrinsicsPowerPC.td:929)
    ppc_altivec_vmaxsd,                        // llvm.ppc.altivec.vmaxsd (IntrinsicsPowerPC.td:932)
    ppc_altivec_vmaxsh,                        // llvm.ppc.altivec.vmaxsh (IntrinsicsPowerPC.td:930)
    ppc_altivec_vmaxsw,                        // llvm.ppc.altivec.vmaxsw (IntrinsicsPowerPC.td:931)
    ppc_altivec_vmaxub,                        // llvm.ppc.altivec.vmaxub (IntrinsicsPowerPC.td:933)
    ppc_altivec_vmaxud,                        // llvm.ppc.altivec.vmaxud (IntrinsicsPowerPC.td:936)
    ppc_altivec_vmaxuh,                        // llvm.ppc.altivec.vmaxuh (IntrinsicsPowerPC.td:934)
    ppc_altivec_vmaxuw,                        // llvm.ppc.altivec.vmaxuw (IntrinsicsPowerPC.td:935)
    ppc_altivec_vmhaddshs,                     // llvm.ppc.altivec.vmhaddshs (IntrinsicsPowerPC.td:971)
    ppc_altivec_vmhraddshs,                    // llvm.ppc.altivec.vmhraddshs (IntrinsicsPowerPC.td:975)
    ppc_altivec_vminfp,                        // llvm.ppc.altivec.vminfp (IntrinsicsPowerPC.td:939)
    ppc_altivec_vminsb,                        // llvm.ppc.altivec.vminsb (IntrinsicsPowerPC.td:940)
    ppc_altivec_vminsd,                        // llvm.ppc.altivec.vminsd (IntrinsicsPowerPC.td:943)
    ppc_altivec_vminsh,                        // llvm.ppc.altivec.vminsh (IntrinsicsPowerPC.td:941)
    ppc_altivec_vminsw,                        // llvm.ppc.altivec.vminsw (IntrinsicsPowerPC.td:942)
    ppc_altivec_vminub,                        // llvm.ppc.altivec.vminub (IntrinsicsPowerPC.td:944)
    ppc_altivec_vminud,                        // llvm.ppc.altivec.vminud (IntrinsicsPowerPC.td:947)
    ppc_altivec_vminuh,                        // llvm.ppc.altivec.vminuh (IntrinsicsPowerPC.td:945)
    ppc_altivec_vminuw,                        // llvm.ppc.altivec.vminuw (IntrinsicsPowerPC.td:946)
    ppc_altivec_vmladduhm,                     // llvm.ppc.altivec.vmladduhm (IntrinsicsPowerPC.td:1089)
    ppc_altivec_vmsumcud,                      // llvm.ppc.altivec.vmsumcud (IntrinsicsPowerPC.td:1009)
    ppc_altivec_vmsummbm,                      // llvm.ppc.altivec.vmsummbm (IntrinsicsPowerPC.td:988)
    ppc_altivec_vmsumshm,                      // llvm.ppc.altivec.vmsumshm (IntrinsicsPowerPC.td:991)
    ppc_altivec_vmsumshs,                      // llvm.ppc.altivec.vmsumshs (IntrinsicsPowerPC.td:994)
    ppc_altivec_vmsumubm,                      // llvm.ppc.altivec.vmsumubm (IntrinsicsPowerPC.td:997)
    ppc_altivec_vmsumudm,                      // llvm.ppc.altivec.vmsumudm (IntrinsicsPowerPC.td:1003)
    ppc_altivec_vmsumuhm,                      // llvm.ppc.altivec.vmsumuhm (IntrinsicsPowerPC.td:1000)
    ppc_altivec_vmsumuhs,                      // llvm.ppc.altivec.vmsumuhs (IntrinsicsPowerPC.td:1006)
    ppc_altivec_vmulesb,                       // llvm.ppc.altivec.vmulesb (IntrinsicsPowerPC.td:1015)
    ppc_altivec_vmulesd,                       // llvm.ppc.altivec.vmulesd (IntrinsicsPowerPC.td:1024)
    ppc_altivec_vmulesh,                       // llvm.ppc.altivec.vmulesh (IntrinsicsPowerPC.td:1018)
    ppc_altivec_vmulesw,                       // llvm.ppc.altivec.vmulesw (IntrinsicsPowerPC.td:1021)
    ppc_altivec_vmuleub,                       // llvm.ppc.altivec.vmuleub (IntrinsicsPowerPC.td:1025)
    ppc_altivec_vmuleud,                       // llvm.ppc.altivec.vmuleud (IntrinsicsPowerPC.td:1034)
    ppc_altivec_vmuleuh,                       // llvm.ppc.altivec.vmuleuh (IntrinsicsPowerPC.td:1028)
    ppc_altivec_vmuleuw,                       // llvm.ppc.altivec.vmuleuw (IntrinsicsPowerPC.td:1031)
    ppc_altivec_vmulhsd,                       // llvm.ppc.altivec.vmulhsd (IntrinsicsPowerPC.td:1338)
    ppc_altivec_vmulhsw,                       // llvm.ppc.altivec.vmulhsw (IntrinsicsPowerPC.td:1336)
    ppc_altivec_vmulhud,                       // llvm.ppc.altivec.vmulhud (IntrinsicsPowerPC.td:1339)
    ppc_altivec_vmulhuw,                       // llvm.ppc.altivec.vmulhuw (IntrinsicsPowerPC.td:1337)
    ppc_altivec_vmulosb,                       // llvm.ppc.altivec.vmulosb (IntrinsicsPowerPC.td:1036)
    ppc_altivec_vmulosd,                       // llvm.ppc.altivec.vmulosd (IntrinsicsPowerPC.td:1045)
    ppc_altivec_vmulosh,                       // llvm.ppc.altivec.vmulosh (IntrinsicsPowerPC.td:1039)
    ppc_altivec_vmulosw,                       // llvm.ppc.altivec.vmulosw (IntrinsicsPowerPC.td:1042)
    ppc_altivec_vmuloub,                       // llvm.ppc.altivec.vmuloub (IntrinsicsPowerPC.td:1046)
    ppc_altivec_vmuloud,                       // llvm.ppc.altivec.vmuloud (IntrinsicsPowerPC.td:1055)
    ppc_altivec_vmulouh,                       // llvm.ppc.altivec.vmulouh (IntrinsicsPowerPC.td:1049)
    ppc_altivec_vmulouw,                       // llvm.ppc.altivec.vmulouw (IntrinsicsPowerPC.td:1052)
    ppc_altivec_vnmsubfp,                      // llvm.ppc.altivec.vnmsubfp (IntrinsicsPowerPC.td:983)
    ppc_altivec_vpdepd,                        // llvm.ppc.altivec.vpdepd (IntrinsicsPowerPC.td:760)
    ppc_altivec_vperm,                         // llvm.ppc.altivec.vperm (IntrinsicsPowerPC.td:1233)
    ppc_altivec_vpextd,                        // llvm.ppc.altivec.vpextd (IntrinsicsPowerPC.td:763)
    ppc_altivec_vpkpx,                         // llvm.ppc.altivec.vpkpx (IntrinsicsPowerPC.td:1094)
    ppc_altivec_vpksdss,                       // llvm.ppc.altivec.vpksdss (IntrinsicsPowerPC.td:1109)
    ppc_altivec_vpksdus,                       // llvm.ppc.altivec.vpksdus (IntrinsicsPowerPC.td:1112)
    ppc_altivec_vpkshss,                       // llvm.ppc.altivec.vpkshss (IntrinsicsPowerPC.td:1097)
    ppc_altivec_vpkshus,                       // llvm.ppc.altivec.vpkshus (IntrinsicsPowerPC.td:1100)
    ppc_altivec_vpkswss,                       // llvm.ppc.altivec.vpkswss (IntrinsicsPowerPC.td:1103)
    ppc_altivec_vpkswus,                       // llvm.ppc.altivec.vpkswus (IntrinsicsPowerPC.td:1106)
    ppc_altivec_vpkudus,                       // llvm.ppc.altivec.vpkudus (IntrinsicsPowerPC.td:1124)
    ppc_altivec_vpkuhus,                       // llvm.ppc.altivec.vpkuhus (IntrinsicsPowerPC.td:1116)
    ppc_altivec_vpkuwus,                       // llvm.ppc.altivec.vpkuwus (IntrinsicsPowerPC.td:1120)
    ppc_altivec_vprtybd,                       // llvm.ppc.altivec.vprtybd (IntrinsicsPowerPC.td:658)
    ppc_altivec_vprtybq,                       // llvm.ppc.altivec.vprtybq (IntrinsicsPowerPC.td:660)
    ppc_altivec_vprtybw,                       // llvm.ppc.altivec.vprtybw (IntrinsicsPowerPC.td:656)
    ppc_altivec_vrefp,                         // llvm.ppc.altivec.vrefp (IntrinsicsPowerPC.td:1251)
    ppc_altivec_vrfim,                         // llvm.ppc.altivec.vrfim (IntrinsicsPowerPC.td:1161)
    ppc_altivec_vrfin,                         // llvm.ppc.altivec.vrfin (IntrinsicsPowerPC.td:1163)
    ppc_altivec_vrfip,                         // llvm.ppc.altivec.vrfip (IntrinsicsPowerPC.td:1165)
    ppc_altivec_vrfiz,                         // llvm.ppc.altivec.vrfiz (IntrinsicsPowerPC.td:1167)
    ppc_altivec_vrlb,                          // llvm.ppc.altivec.vrlb (IntrinsicsPowerPC.td:1220)
    ppc_altivec_vrld,                          // llvm.ppc.altivec.vrld (IntrinsicsPowerPC.td:1224)
    ppc_altivec_vrldmi,                        // llvm.ppc.altivec.vrldmi (IntrinsicsPowerPC.td:1313)
    ppc_altivec_vrldnm,                        // llvm.ppc.altivec.vrldnm (IntrinsicsPowerPC.td:1310)
    ppc_altivec_vrlh,                          // llvm.ppc.altivec.vrlh (IntrinsicsPowerPC.td:1221)
    ppc_altivec_vrlqmi,                        // llvm.ppc.altivec.vrlqmi (IntrinsicsPowerPC.td:1322)
    ppc_altivec_vrlqnm,                        // llvm.ppc.altivec.vrlqnm (IntrinsicsPowerPC.td:1318)
    ppc_altivec_vrlw,                          // llvm.ppc.altivec.vrlw (IntrinsicsPowerPC.td:1222)
    ppc_altivec_vrlwmi,                        // llvm.ppc.altivec.vrlwmi (IntrinsicsPowerPC.td:1306)
    ppc_altivec_vrlwnm,                        // llvm.ppc.altivec.vrlwnm (IntrinsicsPowerPC.td:1303)
    ppc_altivec_vrsqrtefp,                     // llvm.ppc.altivec.vrsqrtefp (IntrinsicsPowerPC.td:1252)
    ppc_altivec_vsel,                          // llvm.ppc.altivec.vsel (IntrinsicsPowerPC.td:1236)
    ppc_altivec_vsl,                           // llvm.ppc.altivec.vsl (IntrinsicsPowerPC.td:1199)
    ppc_altivec_vslb,                          // llvm.ppc.altivec.vslb (IntrinsicsPowerPC.td:1202)
    ppc_altivec_vsldbi,                        // llvm.ppc.altivec.vsldbi (IntrinsicsPowerPC.td:809)
    ppc_altivec_vslh,                          // llvm.ppc.altivec.vslh (IntrinsicsPowerPC.td:1205)
    ppc_altivec_vslo,                          // llvm.ppc.altivec.vslo (IntrinsicsPowerPC.td:1200)
    ppc_altivec_vslv,                          // llvm.ppc.altivec.vslv (IntrinsicsPowerPC.td:1203)
    ppc_altivec_vslw,                          // llvm.ppc.altivec.vslw (IntrinsicsPowerPC.td:1206)
    ppc_altivec_vsr,                           // llvm.ppc.altivec.vsr (IntrinsicsPowerPC.td:1209)
    ppc_altivec_vsrab,                         // llvm.ppc.altivec.vsrab (IntrinsicsPowerPC.td:1215)
    ppc_altivec_vsrah,                         // llvm.ppc.altivec.vsrah (IntrinsicsPowerPC.td:1216)
    ppc_altivec_vsraw,                         // llvm.ppc.altivec.vsraw (IntrinsicsPowerPC.td:1217)
    ppc_altivec_vsrb,                          // llvm.ppc.altivec.vsrb (IntrinsicsPowerPC.td:1212)
    ppc_altivec_vsrdbi,                        // llvm.ppc.altivec.vsrdbi (IntrinsicsPowerPC.td:813)
    ppc_altivec_vsrh,                          // llvm.ppc.altivec.vsrh (IntrinsicsPowerPC.td:1213)
    ppc_altivec_vsro,                          // llvm.ppc.altivec.vsro (IntrinsicsPowerPC.td:1210)
    ppc_altivec_vsrv,                          // llvm.ppc.altivec.vsrv (IntrinsicsPowerPC.td:1204)
    ppc_altivec_vsrw,                          // llvm.ppc.altivec.vsrw (IntrinsicsPowerPC.td:1214)
    ppc_altivec_vstribl,                       // llvm.ppc.altivec.vstribl (IntrinsicsPowerPC.td:770)
    ppc_altivec_vstribl_p,                     // llvm.ppc.altivec.vstribl.p (IntrinsicsPowerPC.td:780)
    ppc_altivec_vstribr,                       // llvm.ppc.altivec.vstribr (IntrinsicsPowerPC.td:768)
    ppc_altivec_vstribr_p,                     // llvm.ppc.altivec.vstribr.p (IntrinsicsPowerPC.td:777)
    ppc_altivec_vstrihl,                       // llvm.ppc.altivec.vstrihl (IntrinsicsPowerPC.td:774)
    ppc_altivec_vstrihl_p,                     // llvm.ppc.altivec.vstrihl.p (IntrinsicsPowerPC.td:786)
    ppc_altivec_vstrihr,                       // llvm.ppc.altivec.vstrihr (IntrinsicsPowerPC.td:772)
    ppc_altivec_vstrihr_p,                     // llvm.ppc.altivec.vstrihr.p (IntrinsicsPowerPC.td:783)
    ppc_altivec_vsubcuq,                       // llvm.ppc.altivec.vsubcuq (IntrinsicsPowerPC.td:967)
    ppc_altivec_vsubcuw,                       // llvm.ppc.altivec.vsubcuw (IntrinsicsPowerPC.td:966)
    ppc_altivec_vsubecuq,                      // llvm.ppc.altivec.vsubecuq (IntrinsicsPowerPC.td:1185)
    ppc_altivec_vsubeuqm,                      // llvm.ppc.altivec.vsubeuqm (IntrinsicsPowerPC.td:1181)
    ppc_altivec_vsubsbs,                       // llvm.ppc.altivec.vsubsbs (IntrinsicsPowerPC.td:961)
    ppc_altivec_vsubshs,                       // llvm.ppc.altivec.vsubshs (IntrinsicsPowerPC.td:963)
    ppc_altivec_vsubsws,                       // llvm.ppc.altivec.vsubsws (IntrinsicsPowerPC.td:965)
    ppc_altivec_vsububs,                       // llvm.ppc.altivec.vsububs (IntrinsicsPowerPC.td:960)
    ppc_altivec_vsubuhs,                       // llvm.ppc.altivec.vsubuhs (IntrinsicsPowerPC.td:962)
    ppc_altivec_vsubuws,                       // llvm.ppc.altivec.vsubuws (IntrinsicsPowerPC.td:964)
    ppc_altivec_vsum2sws,                      // llvm.ppc.altivec.vsum2sws (IntrinsicsPowerPC.td:1061)
    ppc_altivec_vsum4sbs,                      // llvm.ppc.altivec.vsum4sbs (IntrinsicsPowerPC.td:1064)
    ppc_altivec_vsum4shs,                      // llvm.ppc.altivec.vsum4shs (IntrinsicsPowerPC.td:1067)
    ppc_altivec_vsum4ubs,                      // llvm.ppc.altivec.vsum4ubs (IntrinsicsPowerPC.td:1070)
    ppc_altivec_vsumsws,                       // llvm.ppc.altivec.vsumsws (IntrinsicsPowerPC.td:1058)
    ppc_altivec_vupkhpx,                       // llvm.ppc.altivec.vupkhpx (IntrinsicsPowerPC.td:1129)
    ppc_altivec_vupkhsb,                       // llvm.ppc.altivec.vupkhsb (IntrinsicsPowerPC.td:1131)
    ppc_altivec_vupkhsh,                       // llvm.ppc.altivec.vupkhsh (IntrinsicsPowerPC.td:1133)
    ppc_altivec_vupkhsw,                       // llvm.ppc.altivec.vupkhsw (IntrinsicsPowerPC.td:1135)
    ppc_altivec_vupklpx,                       // llvm.ppc.altivec.vupklpx (IntrinsicsPowerPC.td:1137)
    ppc_altivec_vupklsb,                       // llvm.ppc.altivec.vupklsb (IntrinsicsPowerPC.td:1139)
    ppc_altivec_vupklsh,                       // llvm.ppc.altivec.vupklsh (IntrinsicsPowerPC.td:1141)
    ppc_altivec_vupklsw,                       // llvm.ppc.altivec.vupklsw (IntrinsicsPowerPC.td:1143)
    ppc_amo_ldat,                              // llvm.ppc.amo.ldat (IntrinsicsPowerPC.td:2149)
    ppc_amo_lwat,                              // llvm.ppc.amo.lwat (IntrinsicsPowerPC.td:2145)
    ppc_atomic_load_i128,                      // llvm.ppc.atomic.load.i128 (IntrinsicsPowerPC.td:2134)
    ppc_atomic_store_i128,                     // llvm.ppc.atomic.store.i128 (IntrinsicsPowerPC.td:2138)
    ppc_atomicrmw_add_i128,                    // llvm.ppc.atomicrmw.add.i128 (IntrinsicsPowerPC.td:2123)
    ppc_atomicrmw_and_i128,                    // llvm.ppc.atomicrmw.and.i128 (IntrinsicsPowerPC.td:2125)
    ppc_atomicrmw_nand_i128,                   // llvm.ppc.atomicrmw.nand.i128 (IntrinsicsPowerPC.td:2128)
    ppc_atomicrmw_or_i128,                     // llvm.ppc.atomicrmw.or.i128 (IntrinsicsPowerPC.td:2126)
    ppc_atomicrmw_sub_i128,                    // llvm.ppc.atomicrmw.sub.i128 (IntrinsicsPowerPC.td:2124)
    ppc_atomicrmw_xchg_i128,                   // llvm.ppc.atomicrmw.xchg.i128 (IntrinsicsPowerPC.td:2122)
    ppc_atomicrmw_xor_i128,                    // llvm.ppc.atomicrmw.xor.i128 (IntrinsicsPowerPC.td:2127)
    ppc_bcdadd,                                // llvm.ppc.bcdadd (IntrinsicsPowerPC.td:692)
    ppc_bcdadd_p,                              // llvm.ppc.bcdadd.p (IntrinsicsPowerPC.td:696)
    ppc_bcdcopysign,                           // llvm.ppc.bcdcopysign (IntrinsicsPowerPC.td:686)
    ppc_bcdsetsign,                            // llvm.ppc.bcdsetsign (IntrinsicsPowerPC.td:688)
    ppc_bcdsub,                                // llvm.ppc.bcdsub (IntrinsicsPowerPC.td:700)
    ppc_bcdsub_p,                              // llvm.ppc.bcdsub.p (IntrinsicsPowerPC.td:704)
    ppc_bpermd,                                // llvm.ppc.bpermd (IntrinsicsPowerPC.td:83)
    ppc_cbcdtd,                                // llvm.ppc.cbcdtd (IntrinsicsPowerPC.td:680)
    ppc_cbcdtdd,                               // llvm.ppc.cbcdtdd (IntrinsicsPowerPC.td:674)
    ppc_cdtbcd,                                // llvm.ppc.cdtbcd (IntrinsicsPowerPC.td:678)
    ppc_cdtbcdd,                               // llvm.ppc.cdtbcdd (IntrinsicsPowerPC.td:672)
    ppc_cfence,                                // llvm.ppc.cfence (IntrinsicsPowerPC.td:1683)
    ppc_cfuged,                                // llvm.ppc.cfuged (IntrinsicsPowerPC.td:98)
    ppc_cmpb,                                  // llvm.ppc.cmpb (IntrinsicsPowerPC.td:1955)
    ppc_cmpeqb,                                // llvm.ppc.cmpeqb (IntrinsicsPowerPC.td:1943)
    ppc_cmprb,                                 // llvm.ppc.cmprb (IntrinsicsPowerPC.td:1947)
    ppc_cmpxchg_i128,                          // llvm.ppc.cmpxchg.i128 (IntrinsicsPowerPC.td:2129)
    ppc_cntlzdm,                               // llvm.ppc.cntlzdm (IntrinsicsPowerPC.td:104)
    ppc_cnttzdm,                               // llvm.ppc.cnttzdm (IntrinsicsPowerPC.td:108)
    ppc_compare_exp_eq,                        // llvm.ppc.compare.exp.eq (IntrinsicsPowerPC.td:2095)
    ppc_compare_exp_gt,                        // llvm.ppc.compare.exp.gt (IntrinsicsPowerPC.td:2091)
    ppc_compare_exp_lt,                        // llvm.ppc.compare.exp.lt (IntrinsicsPowerPC.td:2087)
    ppc_compare_exp_uo,                        // llvm.ppc.compare.exp.uo (IntrinsicsPowerPC.td:2083)
    ppc_convert_f128_to_ppcf128,               // llvm.ppc.convert.f128.to.ppcf128 (IntrinsicsPowerPC.td:2109)
    ppc_convert_ppcf128_to_f128,               // llvm.ppc.convert.ppcf128.to.f128 (IntrinsicsPowerPC.td:2111)
    ppc_darn,                                  // llvm.ppc.darn (IntrinsicsPowerPC.td:72)
    ppc_darn32,                                // llvm.ppc.darn32 (IntrinsicsPowerPC.td:78)
    ppc_darnraw,                               // llvm.ppc.darnraw (IntrinsicsPowerPC.td:75)
    ppc_dcba,                                  // llvm.ppc.dcba (IntrinsicsPowerPC.td:20)
    ppc_dcbf,                                  // llvm.ppc.dcbf (IntrinsicsPowerPC.td:21)
    ppc_dcbfl,                                 // llvm.ppc.dcbfl (IntrinsicsPowerPC.td:1856)
    ppc_dcbflp,                                // llvm.ppc.dcbflp (IntrinsicsPowerPC.td:1858)
    ppc_dcbfps,                                // llvm.ppc.dcbfps (IntrinsicsPowerPC.td:23)
    ppc_dcbi,                                  // llvm.ppc.dcbi (IntrinsicsPowerPC.td:25)
    ppc_dcbst,                                 // llvm.ppc.dcbst (IntrinsicsPowerPC.td:1860)
    ppc_dcbstps,                               // llvm.ppc.dcbstps (IntrinsicsPowerPC.td:24)
    ppc_dcbt,                                  // llvm.ppc.dcbt (IntrinsicsPowerPC.td:1862)
    ppc_dcbt_with_hint,                        // llvm.ppc.dcbt.with.hint (IntrinsicsPowerPC.td:26)
    ppc_dcbtst,                                // llvm.ppc.dcbtst (IntrinsicsPowerPC.td:1865)
    ppc_dcbtst_with_hint,                      // llvm.ppc.dcbtst.with.hint (IntrinsicsPowerPC.td:28)
    ppc_dcbtstt,                               // llvm.ppc.dcbtstt (IntrinsicsPowerPC.td:1923)
    ppc_dcbtt,                                 // llvm.ppc.dcbtt (IntrinsicsPowerPC.td:1926)
    ppc_dcbz,                                  // llvm.ppc.dcbz (IntrinsicsPowerPC.td:1868)
    ppc_dcbzl,                                 // llvm.ppc.dcbzl (IntrinsicsPowerPC.td:30)
    ppc_divde,                                 // llvm.ppc.divde (IntrinsicsPowerPC.td:57)
    ppc_divdeu,                                // llvm.ppc.divdeu (IntrinsicsPowerPC.td:60)
    ppc_divf128_round_to_odd,                  // llvm.ppc.divf128.round.to.odd (IntrinsicsPowerPC.td:131)
    ppc_divwe,                                 // llvm.ppc.divwe (IntrinsicsPowerPC.td:51)
    ppc_divweu,                                // llvm.ppc.divweu (IntrinsicsPowerPC.td:54)
    ppc_eieio,                                 // llvm.ppc.eieio (IntrinsicsPowerPC.td:1891)
    ppc_extract_exp,                           // llvm.ppc.extract.exp (IntrinsicsPowerPC.td:2013)
    ppc_extract_sig,                           // llvm.ppc.extract.sig (IntrinsicsPowerPC.td:2016)
    ppc_fcfid,                                 // llvm.ppc.fcfid (IntrinsicsPowerPC.td:161)
    ppc_fcfud,                                 // llvm.ppc.fcfud (IntrinsicsPowerPC.td:164)
    ppc_fctid,                                 // llvm.ppc.fctid (IntrinsicsPowerPC.td:167)
    ppc_fctidz,                                // llvm.ppc.fctidz (IntrinsicsPowerPC.td:170)
    ppc_fctiw,                                 // llvm.ppc.fctiw (IntrinsicsPowerPC.td:173)
    ppc_fctiwz,                                // llvm.ppc.fctiwz (IntrinsicsPowerPC.td:176)
    ppc_fctudz,                                // llvm.ppc.fctudz (IntrinsicsPowerPC.td:179)
    ppc_fctuwz,                                // llvm.ppc.fctuwz (IntrinsicsPowerPC.td:182)
    ppc_fence,                                 // llvm.ppc.fence (IntrinsicsPowerPC.td:33)
    ppc_fixed_addr_ld,                         // llvm.ppc.fixed.addr.ld (IntrinsicsPowerPC.td:234)
    ppc_fmaf128_round_to_odd,                  // llvm.ppc.fmaf128.round.to.odd (IntrinsicsPowerPC.td:135)
    ppc_fmsub,                                 // llvm.ppc.fmsub (IntrinsicsPowerPC.td:2038)
    ppc_fmsubs,                                // llvm.ppc.fmsubs (IntrinsicsPowerPC.td:2043)
    ppc_fnabs,                                 // llvm.ppc.fnabs (IntrinsicsPowerPC.td:2102)
    ppc_fnabss,                                // llvm.ppc.fnabss (IntrinsicsPowerPC.td:2105)
    ppc_fnmadd,                                // llvm.ppc.fnmadd (IntrinsicsPowerPC.td:2048)
    ppc_fnmadds,                               // llvm.ppc.fnmadds (IntrinsicsPowerPC.td:2053)
    ppc_fnmsub,                                // llvm.ppc.fnmsub (IntrinsicsPowerPC.td:2058)
    ppc_fre,                                   // llvm.ppc.fre (IntrinsicsPowerPC.td:2063)
    ppc_fres,                                  // llvm.ppc.fres (IntrinsicsPowerPC.td:2066)
    ppc_frsqrte,                               // llvm.ppc.frsqrte (IntrinsicsPowerPC.td:2079)
    ppc_frsqrtes,                              // llvm.ppc.frsqrtes (IntrinsicsPowerPC.td:2081)
    ppc_fsel,                                  // llvm.ppc.fsel (IntrinsicsPowerPC.td:2073)
    ppc_fsels,                                 // llvm.ppc.fsels (IntrinsicsPowerPC.td:2076)
    ppc_get_texasr,                            // llvm.ppc.get.texasr (IntrinsicsPowerPC.td:1650)
    ppc_get_texasru,                           // llvm.ppc.get.texasru (IntrinsicsPowerPC.td:1652)
    ppc_get_tfhar,                             // llvm.ppc.get.tfhar (IntrinsicsPowerPC.td:1654)
    ppc_get_tfiar,                             // llvm.ppc.get.tfiar (IntrinsicsPowerPC.td:1656)
    ppc_icbt,                                  // llvm.ppc.icbt (IntrinsicsPowerPC.td:1870)
    ppc_insert_exp,                            // llvm.ppc.insert.exp (IntrinsicsPowerPC.td:2009)
    ppc_iospace_eieio,                         // llvm.ppc.iospace.eieio (IntrinsicsPowerPC.td:1893)
    ppc_iospace_lwsync,                        // llvm.ppc.iospace.lwsync (IntrinsicsPowerPC.td:1888)
    ppc_iospace_sync,                          // llvm.ppc.iospace.sync (IntrinsicsPowerPC.td:1880)
    ppc_isync,                                 // llvm.ppc.isync (IntrinsicsPowerPC.td:1883)
    ppc_lbarx,                                 // llvm.ppc.lbarx (IntrinsicsPowerPC.td:1895)
    ppc_ldarx,                                 // llvm.ppc.ldarx (IntrinsicsPowerPC.td:1904)
    ppc_lharx,                                 // llvm.ppc.lharx (IntrinsicsPowerPC.td:1898)
    ppc_load2r,                                // llvm.ppc.load2r (IntrinsicsPowerPC.td:1988)
    ppc_load4r,                                // llvm.ppc.load4r (IntrinsicsPowerPC.td:1991)
    ppc_load8r,                                // llvm.ppc.load8r (IntrinsicsPowerPC.td:1995)
    ppc_lwarx,                                 // llvm.ppc.lwarx (IntrinsicsPowerPC.td:1901)
    ppc_lwsync,                                // llvm.ppc.lwsync (IntrinsicsPowerPC.td:1886)
    ppc_maddhd,                                // llvm.ppc.maddhd (IntrinsicsPowerPC.td:1975)
    ppc_maddhdu,                               // llvm.ppc.maddhdu (IntrinsicsPowerPC.td:1979)
    ppc_maddld,                                // llvm.ppc.maddld (IntrinsicsPowerPC.td:1983)
    ppc_maxfe,                                 // llvm.ppc.maxfe (IntrinsicsPowerPC.td:199)
    ppc_maxfl,                                 // llvm.ppc.maxfl (IntrinsicsPowerPC.td:204)
    ppc_maxfs,                                 // llvm.ppc.maxfs (IntrinsicsPowerPC.td:209)
    ppc_mffsl,                                 // llvm.ppc.mffsl (IntrinsicsPowerPC.td:41)
    ppc_mfmsr,                                 // llvm.ppc.mfmsr (IntrinsicsPowerPC.td:1931)
    ppc_mfspr,                                 // llvm.ppc.mfspr (IntrinsicsPowerPC.td:1933)
    ppc_mftbu,                                 // llvm.ppc.mftbu (IntrinsicsPowerPC.td:1929)
    ppc_minfe,                                 // llvm.ppc.minfe (IntrinsicsPowerPC.td:214)
    ppc_minfl,                                 // llvm.ppc.minfl (IntrinsicsPowerPC.td:219)
    ppc_minfs,                                 // llvm.ppc.minfs (IntrinsicsPowerPC.td:224)
    ppc_mma_assemble_acc,                      // llvm.ppc.mma.assemble.acc (IntrinsicsPowerPC.td:1699)
    ppc_mma_build_dmr,                         // llvm.ppc.mma.build.dmr (IntrinsicsPowerPC.td:1747)
    ppc_mma_disassemble_acc,                   // llvm.ppc.mma.disassemble.acc (IntrinsicsPowerPC.td:1704)
    ppc_mma_disassemble_dmr,                   // llvm.ppc.mma.disassemble.dmr (IntrinsicsPowerPC.td:1743)
    ppc_mma_dmmr,                              // llvm.ppc.mma.dmmr (IntrinsicsPowerPC.td:1720)
    ppc_mma_dmsetdmrz,                         // llvm.ppc.mma.dmsetdmrz (IntrinsicsPowerPC.td:1717)
    ppc_mma_dmsha2hash,                        // llvm.ppc.mma.dmsha2hash (IntrinsicsPowerPC.td:1838)
    ppc_mma_dmsha3hash,                        // llvm.ppc.mma.dmsha3hash (IntrinsicsPowerPC.td:1843)
    ppc_mma_dmxor,                             // llvm.ppc.mma.dmxor (IntrinsicsPowerPC.td:1723)
    ppc_mma_dmxvbf16gerx2,                     // llvm.ppc.mma.dmxvbf16gerx2 (IntrinsicsPowerPC.td:284)
    ppc_mma_dmxvbf16gerx2nn,                   // llvm.ppc.mma.dmxvbf16gerx2nn (IntrinsicsPowerPC.td:294)
    ppc_mma_dmxvbf16gerx2np,                   // llvm.ppc.mma.dmxvbf16gerx2np (IntrinsicsPowerPC.td:291)
    ppc_mma_dmxvbf16gerx2pn,                   // llvm.ppc.mma.dmxvbf16gerx2pn (IntrinsicsPowerPC.td:288)
    ppc_mma_dmxvbf16gerx2pp,                   // llvm.ppc.mma.dmxvbf16gerx2pp (IntrinsicsPowerPC.td:285)
    ppc_mma_dmxvf16gerx2,                      // llvm.ppc.mma.dmxvf16gerx2 (IntrinsicsPowerPC.td:284)
    ppc_mma_dmxvf16gerx2nn,                    // llvm.ppc.mma.dmxvf16gerx2nn (IntrinsicsPowerPC.td:294)
    ppc_mma_dmxvf16gerx2np,                    // llvm.ppc.mma.dmxvf16gerx2np (IntrinsicsPowerPC.td:291)
    ppc_mma_dmxvf16gerx2pn,                    // llvm.ppc.mma.dmxvf16gerx2pn (IntrinsicsPowerPC.td:288)
    ppc_mma_dmxvf16gerx2pp,                    // llvm.ppc.mma.dmxvf16gerx2pp (IntrinsicsPowerPC.td:285)
    ppc_mma_dmxvi8gerx4,                       // llvm.ppc.mma.dmxvi8gerx4 (IntrinsicsPowerPC.td:300)
    ppc_mma_dmxvi8gerx4pp,                     // llvm.ppc.mma.dmxvi8gerx4pp (IntrinsicsPowerPC.td:301)
    ppc_mma_dmxvi8gerx4spp,                    // llvm.ppc.mma.dmxvi8gerx4spp (IntrinsicsPowerPC.td:1815)
    ppc_mma_dmxxextfdmr256,                    // llvm.ppc.mma.dmxxextfdmr256 (IntrinsicsPowerPC.td:1735)
    ppc_mma_dmxxextfdmr512,                    // llvm.ppc.mma.dmxxextfdmr512 (IntrinsicsPowerPC.td:1727)
    ppc_mma_dmxxinstdmr256,                    // llvm.ppc.mma.dmxxinstdmr256 (IntrinsicsPowerPC.td:1739)
    ppc_mma_dmxxinstdmr512,                    // llvm.ppc.mma.dmxxinstdmr512 (IntrinsicsPowerPC.td:1731)
    ppc_mma_dmxxshapad,                        // llvm.ppc.mma.dmxxshapad (IntrinsicsPowerPC.td:1847)
    ppc_mma_pmdmxvbf16gerx2,                   // llvm.ppc.mma.pmdmxvbf16gerx2 (IntrinsicsPowerPC.td:284)
    ppc_mma_pmdmxvbf16gerx2nn,                 // llvm.ppc.mma.pmdmxvbf16gerx2nn (IntrinsicsPowerPC.td:294)
    ppc_mma_pmdmxvbf16gerx2np,                 // llvm.ppc.mma.pmdmxvbf16gerx2np (IntrinsicsPowerPC.td:291)
    ppc_mma_pmdmxvbf16gerx2pn,                 // llvm.ppc.mma.pmdmxvbf16gerx2pn (IntrinsicsPowerPC.td:288)
    ppc_mma_pmdmxvbf16gerx2pp,                 // llvm.ppc.mma.pmdmxvbf16gerx2pp (IntrinsicsPowerPC.td:285)
    ppc_mma_pmdmxvf16gerx2,                    // llvm.ppc.mma.pmdmxvf16gerx2 (IntrinsicsPowerPC.td:284)
    ppc_mma_pmdmxvf16gerx2nn,                  // llvm.ppc.mma.pmdmxvf16gerx2nn (IntrinsicsPowerPC.td:294)
    ppc_mma_pmdmxvf16gerx2np,                  // llvm.ppc.mma.pmdmxvf16gerx2np (IntrinsicsPowerPC.td:291)
    ppc_mma_pmdmxvf16gerx2pn,                  // llvm.ppc.mma.pmdmxvf16gerx2pn (IntrinsicsPowerPC.td:288)
    ppc_mma_pmdmxvf16gerx2pp,                  // llvm.ppc.mma.pmdmxvf16gerx2pp (IntrinsicsPowerPC.td:285)
    ppc_mma_pmdmxvi8gerx4,                     // llvm.ppc.mma.pmdmxvi8gerx4 (IntrinsicsPowerPC.td:300)
    ppc_mma_pmdmxvi8gerx4pp,                   // llvm.ppc.mma.pmdmxvi8gerx4pp (IntrinsicsPowerPC.td:301)
    ppc_mma_pmdmxvi8gerx4spp,                  // llvm.ppc.mma.pmdmxvi8gerx4spp (IntrinsicsPowerPC.td:1819)
    ppc_mma_pmxvbf16ger2,                      // llvm.ppc.mma.pmxvbf16ger2 (IntrinsicsPowerPC.td:261)
    ppc_mma_pmxvbf16ger2nn,                    // llvm.ppc.mma.pmxvbf16ger2nn (IntrinsicsPowerPC.td:271)
    ppc_mma_pmxvbf16ger2np,                    // llvm.ppc.mma.pmxvbf16ger2np (IntrinsicsPowerPC.td:268)
    ppc_mma_pmxvbf16ger2pn,                    // llvm.ppc.mma.pmxvbf16ger2pn (IntrinsicsPowerPC.td:265)
    ppc_mma_pmxvbf16ger2pp,                    // llvm.ppc.mma.pmxvbf16ger2pp (IntrinsicsPowerPC.td:262)
    ppc_mma_pmxvf16ger2,                       // llvm.ppc.mma.pmxvf16ger2 (IntrinsicsPowerPC.td:261)
    ppc_mma_pmxvf16ger2nn,                     // llvm.ppc.mma.pmxvf16ger2nn (IntrinsicsPowerPC.td:271)
    ppc_mma_pmxvf16ger2np,                     // llvm.ppc.mma.pmxvf16ger2np (IntrinsicsPowerPC.td:268)
    ppc_mma_pmxvf16ger2pn,                     // llvm.ppc.mma.pmxvf16ger2pn (IntrinsicsPowerPC.td:265)
    ppc_mma_pmxvf16ger2pp,                     // llvm.ppc.mma.pmxvf16ger2pp (IntrinsicsPowerPC.td:262)
    ppc_mma_pmxvf32ger,                        // llvm.ppc.mma.pmxvf32ger (IntrinsicsPowerPC.td:261)
    ppc_mma_pmxvf32gernn,                      // llvm.ppc.mma.pmxvf32gernn (IntrinsicsPowerPC.td:271)
    ppc_mma_pmxvf32gernp,                      // llvm.ppc.mma.pmxvf32gernp (IntrinsicsPowerPC.td:268)
    ppc_mma_pmxvf32gerpn,                      // llvm.ppc.mma.pmxvf32gerpn (IntrinsicsPowerPC.td:265)
    ppc_mma_pmxvf32gerpp,                      // llvm.ppc.mma.pmxvf32gerpp (IntrinsicsPowerPC.td:262)
    ppc_mma_pmxvf64ger,                        // llvm.ppc.mma.pmxvf64ger (IntrinsicsPowerPC.td:261)
    ppc_mma_pmxvf64gernn,                      // llvm.ppc.mma.pmxvf64gernn (IntrinsicsPowerPC.td:271)
    ppc_mma_pmxvf64gernp,                      // llvm.ppc.mma.pmxvf64gernp (IntrinsicsPowerPC.td:268)
    ppc_mma_pmxvf64gerpn,                      // llvm.ppc.mma.pmxvf64gerpn (IntrinsicsPowerPC.td:265)
    ppc_mma_pmxvf64gerpp,                      // llvm.ppc.mma.pmxvf64gerpp (IntrinsicsPowerPC.td:262)
    ppc_mma_pmxvi16ger2,                       // llvm.ppc.mma.pmxvi16ger2 (IntrinsicsPowerPC.td:277)
    ppc_mma_pmxvi16ger2pp,                     // llvm.ppc.mma.pmxvi16ger2pp (IntrinsicsPowerPC.td:278)
    ppc_mma_pmxvi16ger2s,                      // llvm.ppc.mma.pmxvi16ger2s (IntrinsicsPowerPC.td:277)
    ppc_mma_pmxvi16ger2spp,                    // llvm.ppc.mma.pmxvi16ger2spp (IntrinsicsPowerPC.td:278)
    ppc_mma_pmxvi4ger8,                        // llvm.ppc.mma.pmxvi4ger8 (IntrinsicsPowerPC.td:277)
    ppc_mma_pmxvi4ger8pp,                      // llvm.ppc.mma.pmxvi4ger8pp (IntrinsicsPowerPC.td:278)
    ppc_mma_pmxvi8ger4,                        // llvm.ppc.mma.pmxvi8ger4 (IntrinsicsPowerPC.td:277)
    ppc_mma_pmxvi8ger4pp,                      // llvm.ppc.mma.pmxvi8ger4pp (IntrinsicsPowerPC.td:278)
    ppc_mma_pmxvi8ger4spp,                     // llvm.ppc.mma.pmxvi8ger4spp (IntrinsicsPowerPC.td:1805)
    ppc_mma_xvbf16ger2,                        // llvm.ppc.mma.xvbf16ger2 (IntrinsicsPowerPC.td:261)
    ppc_mma_xvbf16ger2nn,                      // llvm.ppc.mma.xvbf16ger2nn (IntrinsicsPowerPC.td:271)
    ppc_mma_xvbf16ger2np,                      // llvm.ppc.mma.xvbf16ger2np (IntrinsicsPowerPC.td:268)
    ppc_mma_xvbf16ger2pn,                      // llvm.ppc.mma.xvbf16ger2pn (IntrinsicsPowerPC.td:265)
    ppc_mma_xvbf16ger2pp,                      // llvm.ppc.mma.xvbf16ger2pp (IntrinsicsPowerPC.td:262)
    ppc_mma_xvf16ger2,                         // llvm.ppc.mma.xvf16ger2 (IntrinsicsPowerPC.td:261)
    ppc_mma_xvf16ger2nn,                       // llvm.ppc.mma.xvf16ger2nn (IntrinsicsPowerPC.td:271)
    ppc_mma_xvf16ger2np,                       // llvm.ppc.mma.xvf16ger2np (IntrinsicsPowerPC.td:268)
    ppc_mma_xvf16ger2pn,                       // llvm.ppc.mma.xvf16ger2pn (IntrinsicsPowerPC.td:265)
    ppc_mma_xvf16ger2pp,                       // llvm.ppc.mma.xvf16ger2pp (IntrinsicsPowerPC.td:262)
    ppc_mma_xvf32ger,                          // llvm.ppc.mma.xvf32ger (IntrinsicsPowerPC.td:261)
    ppc_mma_xvf32gernn,                        // llvm.ppc.mma.xvf32gernn (IntrinsicsPowerPC.td:271)
    ppc_mma_xvf32gernp,                        // llvm.ppc.mma.xvf32gernp (IntrinsicsPowerPC.td:268)
    ppc_mma_xvf32gerpn,                        // llvm.ppc.mma.xvf32gerpn (IntrinsicsPowerPC.td:265)
    ppc_mma_xvf32gerpp,                        // llvm.ppc.mma.xvf32gerpp (IntrinsicsPowerPC.td:262)
    ppc_mma_xvf64ger,                          // llvm.ppc.mma.xvf64ger (IntrinsicsPowerPC.td:261)
    ppc_mma_xvf64gernn,                        // llvm.ppc.mma.xvf64gernn (IntrinsicsPowerPC.td:271)
    ppc_mma_xvf64gernp,                        // llvm.ppc.mma.xvf64gernp (IntrinsicsPowerPC.td:268)
    ppc_mma_xvf64gerpn,                        // llvm.ppc.mma.xvf64gerpn (IntrinsicsPowerPC.td:265)
    ppc_mma_xvf64gerpp,                        // llvm.ppc.mma.xvf64gerpp (IntrinsicsPowerPC.td:262)
    ppc_mma_xvi16ger2,                         // llvm.ppc.mma.xvi16ger2 (IntrinsicsPowerPC.td:277)
    ppc_mma_xvi16ger2pp,                       // llvm.ppc.mma.xvi16ger2pp (IntrinsicsPowerPC.td:278)
    ppc_mma_xvi16ger2s,                        // llvm.ppc.mma.xvi16ger2s (IntrinsicsPowerPC.td:277)
    ppc_mma_xvi16ger2spp,                      // llvm.ppc.mma.xvi16ger2spp (IntrinsicsPowerPC.td:278)
    ppc_mma_xvi4ger8,                          // llvm.ppc.mma.xvi4ger8 (IntrinsicsPowerPC.td:277)
    ppc_mma_xvi4ger8pp,                        // llvm.ppc.mma.xvi4ger8pp (IntrinsicsPowerPC.td:278)
    ppc_mma_xvi8ger4,                          // llvm.ppc.mma.xvi8ger4 (IntrinsicsPowerPC.td:277)
    ppc_mma_xvi8ger4pp,                        // llvm.ppc.mma.xvi8ger4pp (IntrinsicsPowerPC.td:278)
    ppc_mma_xvi8ger4spp,                       // llvm.ppc.mma.xvi8ger4spp (IntrinsicsPowerPC.td:1801)
    ppc_mma_xxmfacc,                           // llvm.ppc.mma.xxmfacc (IntrinsicsPowerPC.td:1711)
    ppc_mma_xxmtacc,                           // llvm.ppc.mma.xxmtacc (IntrinsicsPowerPC.td:1708)
    ppc_mma_xxsetaccz,                         // llvm.ppc.mma.xxsetaccz (IntrinsicsPowerPC.td:1714)
    ppc_mtfsb0,                                // llvm.ppc.mtfsb0 (IntrinsicsPowerPC.td:2019)
    ppc_mtfsb1,                                // llvm.ppc.mtfsb1 (IntrinsicsPowerPC.td:2024)
    ppc_mtfsf,                                 // llvm.ppc.mtfsf (IntrinsicsPowerPC.td:2029)
    ppc_mtfsfi,                                // llvm.ppc.mtfsfi (IntrinsicsPowerPC.td:2033)
    ppc_mtmsr,                                 // llvm.ppc.mtmsr (IntrinsicsPowerPC.td:1935)
    ppc_mtspr,                                 // llvm.ppc.mtspr (IntrinsicsPowerPC.td:1937)
    ppc_mulf128_round_to_odd,                  // llvm.ppc.mulf128.round.to.odd (IntrinsicsPowerPC.td:127)
    ppc_mulhd,                                 // llvm.ppc.mulhd (IntrinsicsPowerPC.td:1959)
    ppc_mulhdu,                                // llvm.ppc.mulhdu (IntrinsicsPowerPC.td:1963)
    ppc_mulhw,                                 // llvm.ppc.mulhw (IntrinsicsPowerPC.td:1967)
    ppc_mulhwu,                                // llvm.ppc.mulhwu (IntrinsicsPowerPC.td:1971)
    ppc_national2packed,                       // llvm.ppc.national2packed (IntrinsicsPowerPC.td:664)
    ppc_pack_longdouble,                       // llvm.ppc.pack.longdouble (IntrinsicsPowerPC.td:67)
    ppc_packed2national,                       // llvm.ppc.packed2national (IntrinsicsPowerPC.td:666)
    ppc_packed2zoned,                          // llvm.ppc.packed2zoned (IntrinsicsPowerPC.td:668)
    ppc_pdepd,                                 // llvm.ppc.pdepd (IntrinsicsPowerPC.td:88)
    ppc_pextd,                                 // llvm.ppc.pextd (IntrinsicsPowerPC.td:92)
    ppc_popcntb,                               // llvm.ppc.popcntb (IntrinsicsPowerPC.td:1874)
    ppc_readflm,                               // llvm.ppc.readflm (IntrinsicsPowerPC.td:38)
    ppc_rldimi,                                // llvm.ppc.rldimi (IntrinsicsPowerPC.td:193)
    ppc_rlwimi,                                // llvm.ppc.rlwimi (IntrinsicsPowerPC.td:185)
    ppc_rlwnm,                                 // llvm.ppc.rlwnm (IntrinsicsPowerPC.td:189)
    ppc_scalar_extract_expq,                   // llvm.ppc.scalar.extract.expq (IntrinsicsPowerPC.td:140)
    ppc_scalar_insert_exp_qp,                  // llvm.ppc.scalar.insert.exp.qp (IntrinsicsPowerPC.td:143)
    ppc_set_texasr,                            // llvm.ppc.set.texasr (IntrinsicsPowerPC.td:1659)
    ppc_set_texasru,                           // llvm.ppc.set.texasru (IntrinsicsPowerPC.td:1661)
    ppc_set_tfhar,                             // llvm.ppc.set.tfhar (IntrinsicsPowerPC.td:1663)
    ppc_set_tfiar,                             // llvm.ppc.set.tfiar (IntrinsicsPowerPC.td:1665)
    ppc_setb,                                  // llvm.ppc.setb (IntrinsicsPowerPC.td:1951)
    ppc_setflm,                                // llvm.ppc.setflm (IntrinsicsPowerPC.td:46)
    ppc_setrnd,                                // llvm.ppc.setrnd (IntrinsicsPowerPC.td:1686)
    ppc_sqrtf128_round_to_odd,                 // llvm.ppc.sqrtf128.round.to.odd (IntrinsicsPowerPC.td:116)
    ppc_stbcx,                                 // llvm.ppc.stbcx (IntrinsicsPowerPC.td:1919)
    ppc_stdcx,                                 // llvm.ppc.stdcx (IntrinsicsPowerPC.td:1908)
    ppc_stfiw,                                 // llvm.ppc.stfiw (IntrinsicsPowerPC.td:1939)
    ppc_sthcx,                                 // llvm.ppc.sthcx (IntrinsicsPowerPC.td:1916)
    ppc_store2r,                               // llvm.ppc.store2r (IntrinsicsPowerPC.td:2000)
    ppc_store4r,                               // llvm.ppc.store4r (IntrinsicsPowerPC.td:2003)
    ppc_store8r,                               // llvm.ppc.store8r (IntrinsicsPowerPC.td:2006)
    ppc_stwcx,                                 // llvm.ppc.stwcx (IntrinsicsPowerPC.td:1912)
    ppc_subf128_round_to_odd,                  // llvm.ppc.subf128.round.to.odd (IntrinsicsPowerPC.td:123)
    ppc_sync,                                  // llvm.ppc.sync (IntrinsicsPowerPC.td:1878)
    ppc_tabort,                                // llvm.ppc.tabort (IntrinsicsPowerPC.td:1630)
    ppc_tabortdc,                              // llvm.ppc.tabortdc (IntrinsicsPowerPC.td:1636)
    ppc_tabortdci,                             // llvm.ppc.tabortdci (IntrinsicsPowerPC.td:1638)
    ppc_tabortwc,                              // llvm.ppc.tabortwc (IntrinsicsPowerPC.td:1632)
    ppc_tabortwci,                             // llvm.ppc.tabortwci (IntrinsicsPowerPC.td:1634)
    ppc_tbegin,                                // llvm.ppc.tbegin (IntrinsicsPowerPC.td:1625)
    ppc_tcheck,                                // llvm.ppc.tcheck (IntrinsicsPowerPC.td:1641)
    ppc_tdw,                                   // llvm.ppc.tdw (IntrinsicsPowerPC.td:149)
    ppc_tend,                                  // llvm.ppc.tend (IntrinsicsPowerPC.td:1627)
    ppc_tendall,                               // llvm.ppc.tendall (IntrinsicsPowerPC.td:1669)
    ppc_test_data_class,                       // llvm.ppc.test.data.class (IntrinsicsPowerPC.td:2099)
    ppc_trap,                                  // llvm.ppc.trap (IntrinsicsPowerPC.td:158)
    ppc_trapd,                                 // llvm.ppc.trapd (IntrinsicsPowerPC.td:155)
    ppc_trechkpt,                              // llvm.ppc.trechkpt (IntrinsicsPowerPC.td:1645)
    ppc_treclaim,                              // llvm.ppc.treclaim (IntrinsicsPowerPC.td:1643)
    ppc_tresume,                               // llvm.ppc.tresume (IntrinsicsPowerPC.td:1671)
    ppc_truncf128_round_to_odd,                // llvm.ppc.truncf128.round.to.odd (IntrinsicsPowerPC.td:113)
    ppc_tsr,                                   // llvm.ppc.tsr (IntrinsicsPowerPC.td:1647)
    ppc_tsuspend,                              // llvm.ppc.tsuspend (IntrinsicsPowerPC.td:1673)
    ppc_ttest,                                 // llvm.ppc.ttest (IntrinsicsPowerPC.td:1676)
    ppc_tw,                                    // llvm.ppc.tw (IntrinsicsPowerPC.td:152)
    ppc_unpack_longdouble,                     // llvm.ppc.unpack.longdouble (IntrinsicsPowerPC.td:64)
    ppc_vsx_assemble_pair,                     // llvm.ppc.vsx.assemble.pair (IntrinsicsPowerPC.td:1691)
    ppc_vsx_disassemble_pair,                  // llvm.ppc.vsx.disassemble.pair (IntrinsicsPowerPC.td:1695)
    ppc_vsx_lxvd2x,                            // llvm.ppc.vsx.lxvd2x (IntrinsicsPowerPC.td:1350)
    ppc_vsx_lxvd2x_be,                         // llvm.ppc.vsx.lxvd2x.be (IntrinsicsPowerPC.td:1356)
    ppc_vsx_lxvl,                              // llvm.ppc.vsx.lxvl (IntrinsicsPowerPC.td:1359)
    ppc_vsx_lxvll,                             // llvm.ppc.vsx.lxvll (IntrinsicsPowerPC.td:1362)
    ppc_vsx_lxvp,                              // llvm.ppc.vsx.lxvp (IntrinsicsPowerPC.td:1365)
    ppc_vsx_lxvprl,                            // llvm.ppc.vsx.lxvprl (IntrinsicsPowerPC.td:1374)
    ppc_vsx_lxvprll,                           // llvm.ppc.vsx.lxvprll (IntrinsicsPowerPC.td:1377)
    ppc_vsx_lxvrl,                             // llvm.ppc.vsx.lxvrl (IntrinsicsPowerPC.td:1368)
    ppc_vsx_lxvrll,                            // llvm.ppc.vsx.lxvrll (IntrinsicsPowerPC.td:1371)
    ppc_vsx_lxvw4x,                            // llvm.ppc.vsx.lxvw4x (IntrinsicsPowerPC.td:1347)
    ppc_vsx_lxvw4x_be,                         // llvm.ppc.vsx.lxvw4x.be (IntrinsicsPowerPC.td:1353)
    ppc_vsx_stxvd2x,                           // llvm.ppc.vsx.stxvd2x (IntrinsicsPowerPC.td:1384)
    ppc_vsx_stxvd2x_be,                        // llvm.ppc.vsx.stxvd2x.be (IntrinsicsPowerPC.td:1388)
    ppc_vsx_stxvl,                             // llvm.ppc.vsx.stxvl (IntrinsicsPowerPC.td:1390)
    ppc_vsx_stxvll,                            // llvm.ppc.vsx.stxvll (IntrinsicsPowerPC.td:1393)
    ppc_vsx_stxvp,                             // llvm.ppc.vsx.stxvp (IntrinsicsPowerPC.td:1396)
    ppc_vsx_stxvprl,                           // llvm.ppc.vsx.stxvprl (IntrinsicsPowerPC.td:1405)
    ppc_vsx_stxvprll,                          // llvm.ppc.vsx.stxvprll (IntrinsicsPowerPC.td:1408)
    ppc_vsx_stxvrl,                            // llvm.ppc.vsx.stxvrl (IntrinsicsPowerPC.td:1399)
    ppc_vsx_stxvrll,                           // llvm.ppc.vsx.stxvrll (IntrinsicsPowerPC.td:1402)
    ppc_vsx_stxvw4x,                           // llvm.ppc.vsx.stxvw4x (IntrinsicsPowerPC.td:1382)
    ppc_vsx_stxvw4x_be,                        // llvm.ppc.vsx.stxvw4x.be (IntrinsicsPowerPC.td:1386)
    ppc_vsx_xsmaxdp,                           // llvm.ppc.vsx.xsmaxdp (IntrinsicsPowerPC.td:1415)
    ppc_vsx_xsmindp,                           // llvm.ppc.vsx.xsmindp (IntrinsicsPowerPC.td:1420)
    ppc_vsx_xvcmpeqdp,                         // llvm.ppc.vsx.xvcmpeqdp (IntrinsicsPowerPC.td:1445)
    ppc_vsx_xvcmpeqdp_p,                       // llvm.ppc.vsx.xvcmpeqdp.p (IntrinsicsPowerPC.td:1448)
    ppc_vsx_xvcmpeqsp,                         // llvm.ppc.vsx.xvcmpeqsp (IntrinsicsPowerPC.td:1452)
    ppc_vsx_xvcmpeqsp_p,                       // llvm.ppc.vsx.xvcmpeqsp.p (IntrinsicsPowerPC.td:1455)
    ppc_vsx_xvcmpgedp,                         // llvm.ppc.vsx.xvcmpgedp (IntrinsicsPowerPC.td:1459)
    ppc_vsx_xvcmpgedp_p,                       // llvm.ppc.vsx.xvcmpgedp.p (IntrinsicsPowerPC.td:1462)
    ppc_vsx_xvcmpgesp,                         // llvm.ppc.vsx.xvcmpgesp (IntrinsicsPowerPC.td:1466)
    ppc_vsx_xvcmpgesp_p,                       // llvm.ppc.vsx.xvcmpgesp.p (IntrinsicsPowerPC.td:1469)
    ppc_vsx_xvcmpgtdp,                         // llvm.ppc.vsx.xvcmpgtdp (IntrinsicsPowerPC.td:1473)
    ppc_vsx_xvcmpgtdp_p,                       // llvm.ppc.vsx.xvcmpgtdp.p (IntrinsicsPowerPC.td:1476)
    ppc_vsx_xvcmpgtsp,                         // llvm.ppc.vsx.xvcmpgtsp (IntrinsicsPowerPC.td:1480)
    ppc_vsx_xvcmpgtsp_p,                       // llvm.ppc.vsx.xvcmpgtsp.p (IntrinsicsPowerPC.td:1483)
    ppc_vsx_xvcvbf16spn,                       // llvm.ppc.vsx.xvcvbf16spn (IntrinsicsPowerPC.td:1553)
    ppc_vsx_xvcvdpsp,                          // llvm.ppc.vsx.xvcvdpsp (IntrinsicsPowerPC.td:1523)
    ppc_vsx_xvcvdpsxws,                        // llvm.ppc.vsx.xvcvdpsxws (IntrinsicsPowerPC.td:1496)
    ppc_vsx_xvcvdpuxws,                        // llvm.ppc.vsx.xvcvdpuxws (IntrinsicsPowerPC.td:1499)
    ppc_vsx_xvcvhpsp,                          // llvm.ppc.vsx.xvcvhpsp (IntrinsicsPowerPC.td:1547)
    ppc_vsx_xvcvspbf16,                        // llvm.ppc.vsx.xvcvspbf16 (IntrinsicsPowerPC.td:1550)
    ppc_vsx_xvcvspdp,                          // llvm.ppc.vsx.xvcvspdp (IntrinsicsPowerPC.td:1514)
    ppc_vsx_xvcvsphp,                          // llvm.ppc.vsx.xvcvsphp (IntrinsicsPowerPC.td:1526)
    ppc_vsx_xvcvspsxds,                        // llvm.ppc.vsx.xvcvspsxds (IntrinsicsPowerPC.td:1502)
    ppc_vsx_xvcvspuxds,                        // llvm.ppc.vsx.xvcvspuxds (IntrinsicsPowerPC.td:1505)
    ppc_vsx_xvcvsxdsp,                         // llvm.ppc.vsx.xvcvsxdsp (IntrinsicsPowerPC.td:1517)
    ppc_vsx_xvcvsxwdp,                         // llvm.ppc.vsx.xvcvsxwdp (IntrinsicsPowerPC.td:1508)
    ppc_vsx_xvcvuxdsp,                         // llvm.ppc.vsx.xvcvuxdsp (IntrinsicsPowerPC.td:1520)
    ppc_vsx_xvcvuxwdp,                         // llvm.ppc.vsx.xvcvuxwdp (IntrinsicsPowerPC.td:1511)
    ppc_vsx_xvdivdp,                           // llvm.ppc.vsx.xvdivdp (IntrinsicsPowerPC.td:1423)
    ppc_vsx_xvdivsp,                           // llvm.ppc.vsx.xvdivsp (IntrinsicsPowerPC.td:1424)
    ppc_vsx_xviexpdp,                          // llvm.ppc.vsx.xviexpdp (IntrinsicsPowerPC.td:1490)
    ppc_vsx_xviexpsp,                          // llvm.ppc.vsx.xviexpsp (IntrinsicsPowerPC.td:1493)
    ppc_vsx_xvmaxdp,                           // llvm.ppc.vsx.xvmaxdp (IntrinsicsPowerPC.td:1413)
    ppc_vsx_xvmaxsp,                           // llvm.ppc.vsx.xvmaxsp (IntrinsicsPowerPC.td:1414)
    ppc_vsx_xvmindp,                           // llvm.ppc.vsx.xvmindp (IntrinsicsPowerPC.td:1418)
    ppc_vsx_xvminsp,                           // llvm.ppc.vsx.xvminsp (IntrinsicsPowerPC.td:1419)
    ppc_vsx_xvrdpip,                           // llvm.ppc.vsx.xvrdpip (IntrinsicsPowerPC.td:1429)
    ppc_vsx_xvredp,                            // llvm.ppc.vsx.xvredp (IntrinsicsPowerPC.td:1435)
    ppc_vsx_xvresp,                            // llvm.ppc.vsx.xvresp (IntrinsicsPowerPC.td:1433)
    ppc_vsx_xvrlw,                             // llvm.ppc.vsx.xvrlw (IntrinsicsPowerPC.td:1223)
    ppc_vsx_xvrspip,                           // llvm.ppc.vsx.xvrspip (IntrinsicsPowerPC.td:1427)
    ppc_vsx_xvrsqrtedp,                        // llvm.ppc.vsx.xvrsqrtedp (IntrinsicsPowerPC.td:1441)
    ppc_vsx_xvrsqrtesp,                        // llvm.ppc.vsx.xvrsqrtesp (IntrinsicsPowerPC.td:1439)
    ppc_vsx_xvtdivdp,                          // llvm.ppc.vsx.xvtdivdp (IntrinsicsPowerPC.td:1566)
    ppc_vsx_xvtdivsp,                          // llvm.ppc.vsx.xvtdivsp (IntrinsicsPowerPC.td:1569)
    ppc_vsx_xvtlsbb,                           // llvm.ppc.vsx.xvtlsbb (IntrinsicsPowerPC.td:1563)
    ppc_vsx_xvtsqrtdp,                         // llvm.ppc.vsx.xvtsqrtdp (IntrinsicsPowerPC.td:1572)
    ppc_vsx_xvtsqrtsp,                         // llvm.ppc.vsx.xvtsqrtsp (IntrinsicsPowerPC.td:1574)
    ppc_vsx_xvtstdcdp,                         // llvm.ppc.vsx.xvtstdcdp (IntrinsicsPowerPC.td:1541)
    ppc_vsx_xvtstdcsp,                         // llvm.ppc.vsx.xvtstdcsp (IntrinsicsPowerPC.td:1544)
    ppc_vsx_xvxexpdp,                          // llvm.ppc.vsx.xvxexpdp (IntrinsicsPowerPC.td:1529)
    ppc_vsx_xvxexpsp,                          // llvm.ppc.vsx.xvxexpsp (IntrinsicsPowerPC.td:1532)
    ppc_vsx_xvxsigdp,                          // llvm.ppc.vsx.xvxsigdp (IntrinsicsPowerPC.td:1535)
    ppc_vsx_xvxsigsp,                          // llvm.ppc.vsx.xvxsigsp (IntrinsicsPowerPC.td:1538)
    ppc_vsx_xxblendvb,                         // llvm.ppc.vsx.xxblendvb (IntrinsicsPowerPC.td:1602)
    ppc_vsx_xxblendvd,                         // llvm.ppc.vsx.xxblendvd (IntrinsicsPowerPC.td:1614)
    ppc_vsx_xxblendvh,                         // llvm.ppc.vsx.xxblendvh (IntrinsicsPowerPC.td:1606)
    ppc_vsx_xxblendvw,                         // llvm.ppc.vsx.xxblendvw (IntrinsicsPowerPC.td:1610)
    ppc_vsx_xxeval,                            // llvm.ppc.vsx.xxeval (IntrinsicsPowerPC.td:1576)
    ppc_vsx_xxextractuw,                       // llvm.ppc.vsx.xxextractuw (IntrinsicsPowerPC.td:1556)
    ppc_vsx_xxgenpcvbm,                        // llvm.ppc.vsx.xxgenpcvbm (IntrinsicsPowerPC.td:1581)
    ppc_vsx_xxgenpcvdm,                        // llvm.ppc.vsx.xxgenpcvdm (IntrinsicsPowerPC.td:1590)
    ppc_vsx_xxgenpcvhm,                        // llvm.ppc.vsx.xxgenpcvhm (IntrinsicsPowerPC.td:1584)
    ppc_vsx_xxgenpcvwm,                        // llvm.ppc.vsx.xxgenpcvwm (IntrinsicsPowerPC.td:1587)
    ppc_vsx_xxinsertw,                         // llvm.ppc.vsx.xxinsertw (IntrinsicsPowerPC.td:1559)
    ppc_vsx_xxleqv,                            // llvm.ppc.vsx.xxleqv (IntrinsicsPowerPC.td:1487)
    ppc_vsx_xxpermx,                           // llvm.ppc.vsx.xxpermx (IntrinsicsPowerPC.td:1595)
    ppc_zoned2packed,                          // llvm.ppc.zoned2packed (IntrinsicsPowerPC.td:670)
}; // enum
} // namespace llvm::Intrinsic
#endif

