/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _SM_INTER_H_RPCGEN
#define _SM_INTER_H_RPCGEN

#define RPCGEN_VERSION	199506

#include <rpc/rpc.h>

#define SM_MAXSTRLEN 1024

struct sm_name {
	char *mon_name;
};
typedef struct sm_name sm_name;
#ifdef __cplusplus
extern "C" bool_t xdr_sm_name(XDR *, sm_name *);
#elif defined(__STDC__)
extern bool_t xdr_sm_name(XDR *, sm_name *);
#else /* Old Style C */
bool_t xdr_sm_name();
#endif /* Old Style C */


struct my_id {
	char *my_name;
	int my_prog;
	int my_vers;
	int my_proc;
};
typedef struct my_id my_id;
#ifdef __cplusplus
extern "C" bool_t xdr_my_id(XDR *, my_id *);
#elif defined(__STDC__)
extern bool_t xdr_my_id(XDR *, my_id *);
#else /* Old Style C */
bool_t xdr_my_id();
#endif /* Old Style C */


struct mon_id {
	char *mon_name;
	struct my_id my_id;
};
typedef struct mon_id mon_id;
#ifdef __cplusplus
extern "C" bool_t xdr_mon_id(XDR *, mon_id *);
#elif defined(__STDC__)
extern bool_t xdr_mon_id(XDR *, mon_id *);
#else /* Old Style C */
bool_t xdr_mon_id();
#endif /* Old Style C */


struct mon {
	struct mon_id mon_id;
	char priv[16];
};
typedef struct mon mon;
#ifdef __cplusplus
extern "C" bool_t xdr_mon(XDR *, mon *);
#elif defined(__STDC__)
extern bool_t xdr_mon(XDR *, mon *);
#else /* Old Style C */
bool_t xdr_mon();
#endif /* Old Style C */


struct stat_chge {
	char *mon_name;
	int state;
};
typedef struct stat_chge stat_chge;
#ifdef __cplusplus
extern "C" bool_t xdr_stat_chge(XDR *, stat_chge *);
#elif defined(__STDC__)
extern bool_t xdr_stat_chge(XDR *, stat_chge *);
#else /* Old Style C */
bool_t xdr_stat_chge();
#endif /* Old Style C */


struct sm_stat {
	int state;
};
typedef struct sm_stat sm_stat;
#ifdef __cplusplus
extern "C" bool_t xdr_sm_stat(XDR *, sm_stat *);
#elif defined(__STDC__)
extern bool_t xdr_sm_stat(XDR *, sm_stat *);
#else /* Old Style C */
bool_t xdr_sm_stat();
#endif /* Old Style C */


enum res {
	stat_succ = 0,
	stat_fail = 1
};
typedef enum res res;
#ifdef __cplusplus
extern "C" bool_t xdr_res(XDR *, res *);
#elif defined(__STDC__)
extern bool_t xdr_res(XDR *, res *);
#else /* Old Style C */
bool_t xdr_res();
#endif /* Old Style C */


struct sm_stat_res {
	res res_stat;
	int state;
};
typedef struct sm_stat_res sm_stat_res;
#ifdef __cplusplus
extern "C" bool_t xdr_sm_stat_res(XDR *, sm_stat_res *);
#elif defined(__STDC__)
extern bool_t xdr_sm_stat_res(XDR *, sm_stat_res *);
#else /* Old Style C */
bool_t xdr_sm_stat_res();
#endif /* Old Style C */


struct status {
	char *mon_name;
	int state;
	char priv[16];
};
typedef struct status status;
#ifdef __cplusplus
extern "C" bool_t xdr_status(XDR *, status *);
#elif defined(__STDC__)
extern bool_t xdr_status(XDR *, status *);
#else /* Old Style C */
bool_t xdr_status();
#endif /* Old Style C */


#define SM_PROG ((u_long)100024)
#define SM_VERS ((u_long)1)

#ifdef __cplusplus
#define SM_STAT ((u_long)1)
extern "C" struct sm_stat_res * sm_stat_1(struct sm_name *, CLIENT *);
extern "C" struct sm_stat_res * sm_stat_1_svc(struct sm_name *, struct svc_req *);
#define SM_MON ((u_long)2)
extern "C" struct sm_stat_res * sm_mon_1(struct mon *, CLIENT *);
extern "C" struct sm_stat_res * sm_mon_1_svc(struct mon *, struct svc_req *);
#define SM_UNMON ((u_long)3)
extern "C" struct sm_stat * sm_unmon_1(struct mon_id *, CLIENT *);
extern "C" struct sm_stat * sm_unmon_1_svc(struct mon_id *, struct svc_req *);
#define SM_UNMON_ALL ((u_long)4)
extern "C" struct sm_stat * sm_unmon_all_1(struct my_id *, CLIENT *);
extern "C" struct sm_stat * sm_unmon_all_1_svc(struct my_id *, struct svc_req *);
#define SM_SIMU_CRASH ((u_long)5)
extern "C" void * sm_simu_crash_1(void *, CLIENT *);
extern "C" void * sm_simu_crash_1_svc(void *, struct svc_req *);
#define SM_NOTIFY ((u_long)6)
extern "C" void * sm_notify_1(struct stat_chge *, CLIENT *);
extern "C" void * sm_notify_1_svc(struct stat_chge *, struct svc_req *);

#elif defined(__STDC__)
#define SM_STAT ((u_long)1)
extern struct sm_stat_res * sm_stat_1(struct sm_name *, CLIENT *);
extern struct sm_stat_res * sm_stat_1_svc(struct sm_name *, struct svc_req *);
#define SM_MON ((u_long)2)
extern struct sm_stat_res * sm_mon_1(struct mon *, CLIENT *);
extern struct sm_stat_res * sm_mon_1_svc(struct mon *, struct svc_req *);
#define SM_UNMON ((u_long)3)
extern struct sm_stat * sm_unmon_1(struct mon_id *, CLIENT *);
extern struct sm_stat * sm_unmon_1_svc(struct mon_id *, struct svc_req *);
#define SM_UNMON_ALL ((u_long)4)
extern struct sm_stat * sm_unmon_all_1(struct my_id *, CLIENT *);
extern struct sm_stat * sm_unmon_all_1_svc(struct my_id *, struct svc_req *);
#define SM_SIMU_CRASH ((u_long)5)
extern void * sm_simu_crash_1(void *, CLIENT *);
extern void * sm_simu_crash_1_svc(void *, struct svc_req *);
#define SM_NOTIFY ((u_long)6)
extern void * sm_notify_1(struct stat_chge *, CLIENT *);
extern void * sm_notify_1_svc(struct stat_chge *, struct svc_req *);

#else /* Old Style C */
#define SM_STAT ((u_long)1)
extern struct sm_stat_res * sm_stat_1();
extern struct sm_stat_res * sm_stat_1_svc();
#define SM_MON ((u_long)2)
extern struct sm_stat_res * sm_mon_1();
extern struct sm_stat_res * sm_mon_1_svc();
#define SM_UNMON ((u_long)3)
extern struct sm_stat * sm_unmon_1();
extern struct sm_stat * sm_unmon_1_svc();
#define SM_UNMON_ALL ((u_long)4)
extern struct sm_stat * sm_unmon_all_1();
extern struct sm_stat * sm_unmon_all_1_svc();
#define SM_SIMU_CRASH ((u_long)5)
extern void * sm_simu_crash_1();
extern void * sm_simu_crash_1_svc();
#define SM_NOTIFY ((u_long)6)
extern void * sm_notify_1();
extern void * sm_notify_1_svc();
#endif /* Old Style C */

#endif /* !_SM_INTER_H_RPCGEN */
