
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_MSP430_NONE,               0)
ELF_RELOC(R_MSP430_32,                 1)
ELF_RELOC(R_MSP430_10_PCREL,           2)
ELF_RELOC(R_MSP430_16,                 3)
ELF_RELOC(R_MSP430_16_PCREL,           4)
ELF_RELOC(R_MSP430_16_BYTE,            5)
ELF_RELOC(R_MSP430_16_PCREL_BYTE,      6)
ELF_RELOC(R_MSP430_2X_PCREL,           7)
ELF_RELOC(R_MSP430_RL_PCREL,           8)
ELF_RELOC(R_MSP430_8,                  9)
ELF_RELOC(R_MSP430_SYM_DIFF,           10)
