#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_AMDGPU_NONE,           0)
ELF_RELOC(R_AMDGPU_ABS32_LO,       1)
ELF_RELOC(R_AMDGPU_ABS32_HI,       2)
ELF_RELOC(R_AMDGPU_ABS64,          3)
ELF_RELOC(R_AMDGPU_REL32,          4)
ELF_RELOC(R_AMDGPU_REL64,          5)
ELF_RELOC(R_AMDGPU_ABS32,          6)
ELF_RELOC(R_AMDGPU_GOTPCREL,       7)
ELF_RELOC(R_AMDGPU_GOTPCREL32_LO,  8)
ELF_RELOC(R_AMDGPU_GOTPCREL32_HI,  9)
ELF_RELOC(R_AMDGPU_REL32_LO,      10)
ELF_RELOC(R_AMDGPU_REL32_HI,      11)
ELF_RELOC(R_AMDGPU_RELATIVE64,    13)
ELF_RELOC(R_AMDGPU_REL16,         14)
